//
// Single ported memory
// (Synchronous)
//
// Freeware September 2015, Fen Logic Ltd.
// This code is free and is delivered 'as is'.
// It comes without any warranty, to the extent permitted by applicable law.
// There are no restrictions to any use or re-use of this code
// in any form or shape. It would be nice if you keep my company name
// in the source or modified source code but even that is not
// required as I can't check it anyway.
// But the code comes with no warranties or guarantees whatsoever.
//
//    This code maps the logic onto a XILINX embedded memory
//
// Known issues:
//    None
//

module spm
#(parameter WIDTH    = 8, // width in bits
            L2DEPTH  = 8, // Log 2 Depth, 8 = 256 deep 
            // Simulation parameters
            CLEAR    = 0,
            FILE     = ""
)
(
   input                  clk,     // system clock 
               
   input                  enable,  // access 
   input                  write,   // write
   input    [L2DEPTH-1:0] address, // read/write address
   input      [WIDTH-1:0] wdata,   // write data
   output reg [WIDTH-1:0] rdata    // read data
);

localparam DEPTH = (1<<L2DEPTH);

reg [WIDTH-1:0] memory [0:DEPTH-1];

   always @(posedge clk)
   begin
      if (enable)
      begin
         if (write)
            memory[address] <= wdata;
         else
            rdata <= memory[address];
      end
   end

/*
  synthesis translate_...
  synopsys  translate_...
  pragma    translate_...
 */
//synopsys translate_off

//
// Direct read and write access operating in
// zero time for testbenches
//

function [WIDTH-1:0] do_read;
input [L2DEPTH-1:0] address;
begin
   do_read = memory[address];
end
endfunction

task do_write;
input [L2DEPTH-1:0] address;
input [WIDTH-1:0] data;
begin
   memory[address] = data;
end
endtask


integer i;
   initial
   begin
      if (CLEAR)
      begin
         for (i=0; i<DEPTH; i=i+1)
            memory[i] = 0;
      end
      #0; // prevent simultaneous clear and file loading 
      // (Yes: it is all blocking code but still I have seen it happen)
      
      // If the filename is not zero use it to load from
      if (FILE[7:0]!=0)
         $readmemh(FILE,memory);
   end
//synopsys translate_on

endmodule